@@ -1,7 +1,9 @@
 #!/bin/bash
-# Copyright 1999-2011 Gentoo Foundation
+# Copyright 1999-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
+source "${PORTAGE_BIN_PATH:-/usr/lib/portage/bin}/eapi.sh"
+
 # We need this next line for "die" and "assert". It expands
 # It _must_ preceed all the calls to die and assert.
 shopt -s expand_aliases
@@ -15,7 +17,7 @@ assert() {
 	done
 }
 
-assert_sigpipe_ok() {
+__assert_sigpipe_ok() {
 	# When extracting a tar file like this:
 	#
 	#     bzip2 -dc foo.tar.bz2 | tar xof -
@@ -43,21 +45,21 @@ assert_sigpipe_ok() {
 
 shopt -s extdebug
 
-# dump_trace([number of funcs on stack to skip],
+# __dump_trace([number of funcs on stack to skip],
 #            [whitespacing for filenames],
 #            [whitespacing for line numbers])
-dump_trace() {
+__dump_trace() {
 	local funcname="" sourcefile="" lineno="" s="yes" n p
 	declare -i strip=${1:-1}
 	local filespacing=$2 linespacing=$3
 
-	# The qa_call() function and anything before it are portage internals
+	# The __qa_call() function and anything before it are portage internals
 	# that the user will not be interested in. Therefore, the stack trace
-	# should only show calls that come after qa_call().
+	# should only show calls that come after __qa_call().
 	(( n = ${#FUNCNAME[@]} - 1 ))
 	(( p = ${#BASH_ARGV[@]} ))
 	while (( n > 0 )) ; do
-		[ "${FUNCNAME[${n}]}" == "qa_call" ] && break
+		[ "${FUNCNAME[${n}]}" == "__qa_call" ] && break
 		(( p -= ${BASH_ARGC[${n}]} ))
 		(( n-- ))
 	done
@@ -86,7 +88,7 @@ dump_trace() {
 }
 
 nonfatal() {
-	if has "${EAPI:-0}" 0 1 2 3 3_pre2 ; then
+	if ! ___eapi_has_nonfatal; then
 		die "$FUNCNAME() not supported in this EAPI"
 	fi
 	if [[ $# -lt 1 ]]; then
@@ -96,18 +98,24 @@ nonfatal() {
 	PORTAGE_NONFATAL=1 "$@"
 }
 
-helpers_die() {
-	case "${EAPI:-0}" in
-		0|1|2|3)
-			echo -e "$@" >&2
-			;;
-		*)
-			die "$@"
-			;;
-	esac
+__bashpid() {
+	# The BASHPID variable is new to bash-4.0, so add a hack for older
+	# versions.  This must be used like so:
+	# ${BASHPID:-$(__bashpid)}
+	sh -c 'echo ${PPID}'
+}
+
+__helpers_die() {
+	if ___eapi_helpers_can_die; then
+		die "$@"
+	else
+		echo -e "$@" >&2
+	fi
 }
 
 die() {
+	local IFS=$' \t\n'
+
 	if [[ $PORTAGE_NONFATAL -eq 1 ]]; then
 		echo -e " $WARN*$NORMAL ${FUNCNAME[1]}: WARNING: $@" >&2
 		return 1
@@ -124,7 +132,7 @@ die() {
 	# setup spacing to make output easier to read
 	(( n = ${#FUNCNAME[@]} - 1 ))
 	while (( n > 0 )) ; do
-		[ "${FUNCNAME[${n}]}" == "qa_call" ] && break
+		[ "${FUNCNAME[${n}]}" == "__qa_call" ] && break
 		(( n-- ))
 	done
 	(( n == 0 )) && (( n = ${#FUNCNAME[@]} - 1 ))
@@ -140,14 +148,14 @@ die() {
 	# get a stack trace, so at least report the phase that failed.
 	local phase_str=
 	[[ -n $EBUILD_PHASE ]] && phase_str=" ($EBUILD_PHASE phase)"
-	eerror "ERROR: $CATEGORY/$PF failed${phase_str}:"
+	eerror "ERROR: ${CATEGORY}/${PF}::${PORTAGE_REPO_NAME} failed${phase_str}:"
 	eerror "  ${*:-(no error message)}"
 	eerror
-	# dump_trace is useless when the main script is a helper binary
+	# __dump_trace is useless when the main script is a helper binary
 	local main_index
 	(( main_index = ${#BASH_SOURCE[@]} - 1 ))
 	if has ${BASH_SOURCE[$main_index]##*/} ebuild.sh misc-functions.sh ; then
-	dump_trace 2 ${filespacing} ${linespacing}
+	__dump_trace 2 ${filespacing} ${linespacing}
 	eerror "  $(printf "%${filespacing}s" "${BASH_SOURCE[1]##*/}"), line $(printf "%${linespacing}s" "${BASH_LINENO[0]}"):  Called die"
 	eerror "The specific snippet of code:"
 	# This scans the file that called die and prints out the logic that
@@ -173,37 +181,15 @@ die() {
 		| while read -r n ; do eerror "  ${n#RETAIN-LEADING-SPACE}" ; done
 	eerror
 	fi
-	eerror "If you need support, post the output of 'emerge --info =$CATEGORY/$PF',"
-	eerror "the complete build log and the output of 'emerge -pqv =$CATEGORY/$PF'."
-	if [[ -n ${EBUILD_OVERLAY_ECLASSES} ]] ; then
-		eerror "This ebuild used the following eclasses from overlays:"
-		local x
-		for x in ${EBUILD_OVERLAY_ECLASSES} ; do
-			eerror "  ${x}"
-		done
-	fi
-	if [ "${EMERGE_FROM}" != "binary" ] && \
-		! hasq ${EBUILD_PHASE} prerm postrm && \
-		[ "${EBUILD#${PORTDIR}/}" == "${EBUILD}" ] ; then
-		local overlay=${EBUILD%/*}
-		overlay=${overlay%/*}
-		overlay=${overlay%/*}
-		if [[ -n $PORTAGE_REPO_NAME ]] ; then
-			eerror "This ebuild is from an overlay named" \
-				"'$PORTAGE_REPO_NAME': '${overlay}/'"
-		else
-			eerror "This ebuild is from an overlay: '${overlay}/'"
-		fi
-	elif [[ -n $PORTAGE_REPO_NAME && -f "$PORTDIR"/profiles/repo_name ]] ; then
-		local portdir_repo_name=$(<"$PORTDIR"/profiles/repo_name)
-		if [[ -n $portdir_repo_name && \
-			$portdir_repo_name != $PORTAGE_REPO_NAME ]] ; then
-			eerror "This ebuild is from a repository" \
-				"named '$PORTAGE_REPO_NAME'"
-		fi
-	fi
+	eerror "If you need support, post the output of \`emerge --info '=${CATEGORY}/${PF}::${PORTAGE_REPO_NAME}'\`,"
+	eerror "the complete build log and the output of \`emerge -pqv '=${CATEGORY}/${PF}::${PORTAGE_REPO_NAME}'\`."
 
-	if [[ "${EBUILD_PHASE/depend}" == "${EBUILD_PHASE}" ]] ; then
+	# Only call die hooks here if we are executed via ebuild.sh or
+	# misc-functions.sh, since those are the only cases where the environment
+	# contains the hook functions. When necessary (like for __helpers_die), die
+	# hooks are automatically called later by a misc-functions.sh invocation.
+	if has ${BASH_SOURCE[$main_index]##*/} ebuild.sh misc-functions.sh && \
+		[[ ${EBUILD_PHASE} != depend ]] ; then
 		local x
 		for x in $EBUILD_DEATH_HOOKS; do
 			${x} "$@" >&2 1>&2
@@ -211,8 +197,16 @@ die() {
 		> "$PORTAGE_BUILDDIR/.die_hooks"
 	fi
 
-	[[ -n ${PORTAGE_LOG_FILE} ]] \
-		&& eerror "The complete build log is located at '${PORTAGE_LOG_FILE}'."
+	if [[ -n ${PORTAGE_LOG_FILE} ]] ; then
+		eerror "The complete build log is located at '${PORTAGE_LOG_FILE}'."
+		if [[ ${PORTAGE_LOG_FILE} != ${T}/* ]] && \
+			! has fail-clean ${FEATURES} ; then
+			# Display path to symlink in ${T}, as requested in bug #412865.
+			local log_ext=log
+			[[ ${PORTAGE_LOG_FILE} != *.log ]] && log_ext+=.${PORTAGE_LOG_FILE##*.}
+			eerror "For convenience, a symlink to the build log is located at '${T}/build.${log_ext}'."
+		fi
+	fi
 	if [ -f "${T}/environment" ] ; then
 		eerror "The ebuild environment file is located at '${T}/environment'."
 	elif [ -d "${T}" ] ; then
@@ -222,32 +216,27 @@ die() {
 		} > "${T}/die.env"
 		eerror "The ebuild environment file is located at '${T}/die.env'."
 	fi
+	eerror "Working directory: '$(pwd)'"
 	eerror "S: '${S}'"
 
 	[[ -n $PORTAGE_EBUILD_EXIT_FILE ]] && > "$PORTAGE_EBUILD_EXIT_FILE"
 	[[ -n $PORTAGE_IPC_DAEMON ]] && "$PORTAGE_BIN_PATH"/ebuild-ipc exit 1
 
 	# subshell die support
-	[[ $BASHPID = $EBUILD_MASTER_PID ]] || kill -s SIGTERM $EBUILD_MASTER_PID
+	[[ ${BASHPID:-$(__bashpid)} == ${EBUILD_MASTER_PID} ]] || kill -s SIGTERM ${EBUILD_MASTER_PID}
 	exit 1
 }
 
-# We need to implement diefunc() since environment.bz2 files contain
-# calls to it (due to alias expansion).
-diefunc() {
-	die "${@}"
-}
-
-quiet_mode() {
+__quiet_mode() {
 	[[ ${PORTAGE_QUIET} -eq 1 ]]
 }
 
-vecho() {
-	quiet_mode || echo "$@"
+__vecho() {
+	__quiet_mode || echo "$@"
 }
 
 # Internal logging function, don't use this in ebuilds
-elog_base() {
+__elog_base() {
 	local messagetype
 	[ -z "${1}" -o -z "${T}" -o ! -d "${T}/logging" ] && return 1
 	case "${1}" in
@@ -256,7 +245,7 @@ elog_base() {
 			shift
 			;;
 		*)
-			vecho -e " ${BAD}*${NORMAL} Invalid use of internal function elog_base(), next message will not be logged"
+			__vecho -e " ${BAD}*${NORMAL} Invalid use of internal function __elog_base(), next message will not be logged"
 			return 1
 			;;
 	esac
@@ -268,17 +257,17 @@ elog_base() {
 }
 
 eqawarn() {
-	elog_base QA "$*"
+	__elog_base QA "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -e "$@" | while read -r ; do
-		vecho " $WARN*$NORMAL $REPLY" >&2
+		__vecho " $WARN*$NORMAL $REPLY" >&2
 	done
 	LAST_E_CMD="eqawarn"
 	return 0
 }
 
 elog() {
-	elog_base LOG "$*"
+	__elog_base LOG "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -e "$@" | while read -r ; do
 		echo " $GOOD*$NORMAL $REPLY"
@@ -287,26 +276,8 @@ elog() {
 	return 0
 }
 
-esyslog() {
-	local pri=
-	local tag=
-
-	if [ -x /usr/bin/logger ]
-	then
-		pri="$1"
-		tag="$2"
-
-		shift 2
-		[ -z "$*" ] && return 0
-
-		/usr/bin/logger -p "${pri}" -t "${tag}" -- "$*"
-	fi
-
-	return 0
-}
-
 einfo() {
-	elog_base INFO "$*"
+	__elog_base INFO "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -e "$@" | while read -r ; do
 		echo " $GOOD*$NORMAL $REPLY"
@@ -316,7 +287,7 @@ einfo() {
 }
 
 einfon() {
-	elog_base INFO "$*"
+	__elog_base INFO "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -ne " ${GOOD}*${NORMAL} $*"
 	LAST_E_CMD="einfon"
@@ -324,7 +295,7 @@ einfon() {
 }
 
 ewarn() {
-	elog_base WARN "$*"
+	__elog_base WARN "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -e "$@" | while read -r ; do
 		echo " $WARN*$NORMAL $RC_INDENTATION$REPLY" >&2
@@ -334,7 +305,7 @@ ewarn() {
 }
 
 eerror() {
-	elog_base ERROR "$*"
+	__elog_base ERROR "$*"
 	[[ ${RC_ENDCOL} != "yes" && ${LAST_E_CMD} == "ebegin" ]] && echo
 	echo -e "$@" | while read -r ; do
 		echo " $BAD*$NORMAL $RC_INDENTATION$REPLY" >&2
@@ -359,7 +330,7 @@ ebegin() {
 	return 0
 }
 
-_eend() {
+__eend() {
 	local retval=${1:-0} efunc=${2:-eerror} msg
 	shift 2
 
@@ -386,64 +357,13 @@ eend() {
 	local retval=${1:-0}
 	shift
 
-	_eend ${retval} eerror "$*"
+	__eend ${retval} eerror "$*"
 
 	LAST_E_CMD="eend"
 	return ${retval}
 }
 
-KV_major() {
-	[[ -z $1 ]] && return 1
-
-	local KV=$@
-	echo "${KV%%.*}"
-}
-
-KV_minor() {
-	[[ -z $1 ]] && return 1
-
-	local KV=$@
-	KV=${KV#*.}
-	echo "${KV%%.*}"
-}
-
-KV_micro() {
-	[[ -z $1 ]] && return 1
-
-	local KV=$@
-	KV=${KV#*.*.}
-	echo "${KV%%[^[:digit:]]*}"
-}
-
-KV_to_int() {
-	[[ -z $1 ]] && return 1
-
-	local KV_MAJOR=$(KV_major "$1")
-	local KV_MINOR=$(KV_minor "$1")
-	local KV_MICRO=$(KV_micro "$1")
-	local KV_int=$(( KV_MAJOR * 65536 + KV_MINOR * 256 + KV_MICRO ))
-
-	# We make version 2.2.0 the minimum version we will handle as
-	# a sanity check ... if its less, we fail ...
-	if [[ ${KV_int} -ge 131584 ]] ; then
-		echo "${KV_int}"
-		return 0
-	fi
-
-	return 1
-}
-
-_RC_GET_KV_CACHE=""
-get_KV() {
-	[[ -z ${_RC_GET_KV_CACHE} ]] \
-		&& _RC_GET_KV_CACHE=$(uname -r)
-
-	echo $(KV_to_int "${_RC_GET_KV_CACHE}")
-
-	return $?
-}
-
-unset_colors() {
+__unset_colors() {
 	COLS=80
 	ENDCOL=
 
@@ -455,9 +375,15 @@ unset_colors() {
 	BRACKET=
 }
 
-set_colors() {
+__set_colors() {
 	COLS=${COLUMNS:-0}      # bash's internal COLUMNS variable
-	(( COLS == 0 )) && COLS=$(set -- $(stty size 2>/dev/null) ; echo $2)
+	# Avoid wasteful stty calls during the "depend" phases.
+	# If stdout is a pipe, the parent process can export COLUMNS
+	# if it's relevant. Use an extra subshell for stty calls, in
+	# order to redirect "/dev/tty: No such device or address"
+	# error from bash to /dev/null.
+	[[ $COLS == 0 && $EBUILD_PHASE != depend ]] && \
+		COLS=$(set -- $( ( stty size </dev/tty ) 2>/dev/null || echo 24 80 ) ; echo $2)
 	(( COLS > 0 )) || (( COLS = 80 ))
 
 	# Now, ${ENDCOL} will move us to the end of the
@@ -471,8 +397,8 @@ set_colors() {
 		BAD=$'\e[31;01m'
 		HILITE=$'\e[36;01m'
 		BRACKET=$'\e[34;01m'
+		NORMAL=$'\e[0m'
 	fi
-	NORMAL=$'\e[0m'
 }
 
 RC_ENDCOL="yes"
@@ -482,10 +408,10 @@ RC_DOT_PATTERN=''
 
 case "${NOCOLOR:-false}" in
 	yes|true)
-		unset_colors
+		__unset_colors
 		;;
 	no|false)
-		set_colors
+		__set_colors
 		;;
 esac
 
@@ -511,111 +437,49 @@ if [[ -z ${XARGS} ]] ; then
 	esac
 fi
 
-has() {
-	hasq "$@"
+hasq() {
+	has $EBUILD_PHASE prerm postrm || eqawarn \
+		"QA Notice: The 'hasq' function is deprecated (replaced by 'has')"
+	has "$@"
 }
 
 hasv() {
-	if hasq "$@" ; then
+	if has "$@" ; then
 		echo "$1"
 		return 0
 	fi
 	return 1
 }
 
-hasq() {
-	[[ " ${*:2} " == *" $1 "* ]]
+has() {
+	local needle=$1
+	shift
+
+	local x
+	for x in "$@"; do
+		[ "${x}" = "${needle}" ] && return 0
+	done
+	return 1
 }
 
-# @FUNCTION: save_ebuild_env
-# @DESCRIPTION:
-# echo the current environment to stdout, filtering out redundant info.
-#
-# --exclude-init-phases causes pkg_nofetch and src_* phase functions to
-# be excluded from the output. These function are not needed for installation
-# or removal of the packages, and can therefore be safely excluded.
-#
-save_ebuild_env() {
-	(
-
-		if hasq --exclude-init-phases $* ; then
-			unset S _E_DOCDESTTREE_ _E_EXEDESTTREE_
-			if [[ -n $PYTHONPATH ]] ; then
-				export PYTHONPATH=${PYTHONPATH/${PORTAGE_PYM_PATH}:}
-				[[ -z $PYTHONPATH ]] && unset PYTHONPATH
-			fi
+__repo_key() {
+	local appropriate_section=0 exit_status=1 line saved_extglob_shopt=$(shopt -p extglob)
+	shopt -s extglob
+	while read line; do
+		[[ ${appropriate_section} == 0 && ${line} == "[$1]" ]] && appropriate_section=1 && continue
+		[[ ${appropriate_section} == 1 && ${line} == "["*"]" ]] && appropriate_section=0 && continue
+		# If a conditional expression like [[ ${line} == $2*( )=* ]] is used
+		# then bash-3.2 produces an error like the following when the file is
+		# sourced: syntax error in conditional expression: unexpected token `('
+		# Therefore, use a regular expression for compatibility.
+		if [[ ${appropriate_section} == 1 && ${line} =~ ^${2}[[:space:]]*= ]]; then
+			echo "${line##$2*( )=*( )}"
+			exit_status=0
+			break
 		fi
-
-		# misc variables inherited from the calling environment
-		unset COLORTERM DISPLAY EDITOR LESS LESSOPEN LOGNAME LS_COLORS PAGER \
-			TERM TERMCAP USER ftp_proxy http_proxy no_proxy
-
-		# other variables inherited from the calling environment
-		unset CVS_RSH ECHANGELOG_USER GPG_AGENT_INFO \
-		SSH_AGENT_PID SSH_AUTH_SOCK STY WINDOW XAUTHORITY
-
-		# CCACHE and DISTCC config
-		unset ${!CCACHE_*} ${!DISTCC_*}
-
-		# There's no need to bloat environment.bz2 with internally defined
-		# functions and variables, so filter them out if possible.
-
-		for x in pkg_setup pkg_nofetch src_unpack src_prepare src_configure \
-			src_compile src_test src_install pkg_preinst pkg_postinst \
-			pkg_prerm pkg_postrm ; do
-			unset -f default_$x _eapi{0,1,2,3,4}_$x
-		done
-		unset x
-
-		unset -f assert assert_sigpipe_ok dump_trace die diefunc \
-			quiet_mode vecho elog_base eqawarn elog \
-			esyslog einfo einfon ewarn eerror ebegin _eend eend KV_major \
-			KV_minor KV_micro KV_to_int get_KV unset_colors set_colors has \
-			has_phase_defined_up_to \
-			hasg hasgq hasv hasq qa_source qa_call \
-			addread addwrite adddeny addpredict _sb_append_var \
-			lchown lchgrp esyslog use usev useq has_version portageq \
-			best_version use_with use_enable register_die_hook \
-			keepdir unpack strip_duplicate_slashes econf einstall \
-			dyn_setup dyn_unpack dyn_clean into insinto exeinto docinto \
-			insopts diropts exeopts libopts docompress \
-			abort_handler abort_prepare abort_configure abort_compile \
-			abort_test abort_install dyn_prepare dyn_configure \
-			dyn_compile dyn_test dyn_install \
-			dyn_preinst dyn_help debug-print debug-print-function \
-			debug-print-section inherit EXPORT_FUNCTIONS remove_path_entry \
-			save_ebuild_env filter_readonly_variables preprocess_ebuild_env \
-			source_all_bashrcs ebuild_main \
-			ebuild_phase ebuild_phase_with_hooks \
-			_ebuild_arg_to_phase _ebuild_phase_funcs default \
-			_pipestatus \
-			${QA_INTERCEPTORS}
-
-		# portage config variables and variables set directly by portage
-		unset ACCEPT_LICENSE BAD BRACKET BUILD_PREFIX COLS \
-			DISTCC_DIR DISTDIR DOC_SYMLINKS_DIR \
-			EBUILD_FORCE_TEST EBUILD_MASTER_PID \
-			ECLASS_DEPTH ENDCOL FAKEROOTKEY \
-			GOOD HILITE HOME \
-			LAST_E_CMD LAST_E_LEN LD_PRELOAD MISC_FUNCTIONS_ARGS MOPREFIX \
-			NOCOLOR NORMAL PKGDIR PKGUSE PKG_LOGDIR PKG_TMPDIR \
-			PORTAGE_BASHRCS_SOURCED PORTAGE_NONFATAL PORTAGE_QUIET \
-			PORTAGE_SANDBOX_DENY PORTAGE_SANDBOX_PREDICT \
-			PORTAGE_SANDBOX_READ PORTAGE_SANDBOX_WRITE PREROOTPATH \
-			QA_INTERCEPTORS \
-			RC_DEFAULT_INDENT RC_DOT_PATTERN RC_ENDCOL RC_INDENTATION  \
-			ROOT ROOTPATH RPMDIR TEMP TMP TMPDIR USE_EXPAND \
-			WARN XARGS _RC_GET_KV_CACHE
-
-		# user config variables
-		unset DOC_SYMLINKS_DIR INSTALL_MASK PKG_INSTALL_MASK
-
-		declare -p
-		declare -fp
-		if [[ ${BASH_VERSINFO[0]} == 3 ]]; then
-			export
-		fi
-	)
+	done <<< "${PORTAGE_REPOSITORIES}"
+	eval "${saved_extglob_shopt}"
+	return ${exit_status}
 }
 
 true
