@@ -1,6 +1,5 @@
 # Copyright 2007 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: profiles.py 14540 2009-10-10 01:58:52Z zmedico $
 
 import logging
 
@@ -34,7 +33,7 @@ class PackagesSystemSet(PackageSet):
 			writemsg_level("\nPackagesSystemSet: profile paths: %s\n" % \
 				(self._profile_paths,), level=logging.DEBUG, noiselevel=-1)
 
-		mylist = [grabfile_package(os.path.join(x, "packages")) for x in self._profile_paths]
+		mylist = [grabfile_package(os.path.join(x, "packages"), verify_eapi=True) for x in self._profile_paths]
 
 		if debug:
 			writemsg_level("\nPackagesSystemSet: raw packages: %s\n" % \
