@@ -1,9 +1,9 @@
-# Copyright 1998-2004 Gentoo Foundation
+# Copyright 1998-2014 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: exception.py 15237 2010-01-29 18:45:49Z zmedico $
 
+import signal
 import sys
-from portage import _unicode_encode
+from portage import _encodings, _unicode_encode, _unicode_decode
 from portage.localization import _
 
 if sys.hexversion >= 0x3000000:
@@ -11,23 +11,44 @@ if sys.hexversion >= 0x3000000:
 
 class PortageException(Exception):
 	"""General superclass for portage exceptions"""
-	def __init__(self,value):
-		self.value = value[:]
-		if sys.hexversion < 0x3000000 and isinstance(self.value, unicode):
-			# Workaround for string formatting operator and unicode value
-			# attribute triggering empty output in formatted string.
-			self.value = _unicode_encode(self.value)
-	def __str__(self):
-		if isinstance(self.value, basestring):
-			return self.value
-		else:
-			return repr(self.value)
+	if sys.hexversion >= 0x3000000:
+		def __init__(self,value):
+			self.value = value[:]
+
+		def __str__(self):
+			if isinstance(self.value, str):
+				return self.value
+			else:
+				return repr(self.value)
+	else:
+		def __init__(self,value):
+			self.value = value[:]
+			if isinstance(self.value, basestring):
+				self.value = _unicode_decode(self.value,
+					encoding=_encodings['content'], errors='replace')
+
+		def __unicode__(self):
+			if isinstance(self.value, unicode):
+				return self.value
+			else:
+				return _unicode_decode(repr(self.value),
+					encoding=_encodings['content'], errors='replace')
+
+		def __str__(self):
+			if isinstance(self.value, unicode):
+				return _unicode_encode(self.value,
+					encoding=_encodings['content'], errors='backslashreplace')
+			else:
+				return repr(self.value)
 
 class CorruptionError(PortageException):
 	"""Corruption indication"""
 
 class InvalidDependString(PortageException):
 	"""An invalid depend string has been encountered"""
+	def __init__(self, value, errors=None):
+		PortageException.__init__(self, value)
+		self.errors = errors
 
 class InvalidVersionString(PortageException):
 	"""An invalid version string has been encountered"""
@@ -46,6 +67,9 @@ class ParseError(PortageException):
 
 class InvalidData(PortageException):
 	"""An incorrect formatting was passed instead of the expected one"""
+	def __init__(self, value, category=None):
+		PortageException.__init__(self, value)
+		self.category = category
 
 class InvalidDataType(PortageException):
 	"""An incorrect type was passed instead of the expected one"""
@@ -63,6 +87,10 @@ class OperationNotPermitted(PortageException):
 	from errno import EPERM as errno
 	"""An operation was not permitted operating system"""
 
+class OperationNotSupported(PortageException):
+	from errno import EOPNOTSUPP as errno
+	"""Operation not supported"""
+
 class PermissionDenied(PortageException):
 	from errno import EACCES as errno
 	"""Permission denied"""
@@ -71,6 +99,33 @@ class TryAgain(PortageException):
 	from errno import EAGAIN as errno
 	"""Try again"""
 
+class TimeoutException(PortageException):
+	"""Operation timed out"""
+	# NOTE: ETIME is undefined on FreeBSD (bug #336875)
+	#from errno import ETIME as errno
+
+class AlarmSignal(TimeoutException):
+	def __init__(self, value, signum=None, frame=None):
+		TimeoutException.__init__(self, value)
+		self.signum = signum
+		self.frame = frame
+
+	@classmethod
+	def register(cls, time):
+		signal.signal(signal.SIGALRM, cls._signal_handler)
+		signal.alarm(time)
+
+	@classmethod
+	def unregister(cls):
+		signal.alarm(0)
+		signal.signal(signal.SIGALRM, signal.SIG_DFL)
+
+	@classmethod
+	def _signal_handler(cls, signum, frame):
+		signal.signal(signal.SIGALRM, signal.SIG_DFL)
+		raise AlarmSignal("alarm signal",
+			signum=signum, frame=frame)
+
 class ReadOnlyFileSystem(PortageException):
 	"""Read-only file system"""
 
@@ -99,19 +154,33 @@ class InvalidPackageName(PortagePackageException):
 
 class InvalidAtom(PortagePackageException):
 	"""Malformed atom spec"""
+	def __init__(self, value, category=None):
+		PortagePackageException.__init__(self, value)
+		self.category = category
 
 class UnsupportedAPIException(PortagePackageException):
 	"""Unsupported API"""
 	def __init__(self, cpv, eapi):
 		self.cpv, self.eapi = cpv, eapi
 	def __str__(self):
+		eapi = self.eapi
+		if not isinstance(eapi, basestring):
+			eapi = str(eapi)
+		eapi = eapi.lstrip("-")
 		msg = _("Unable to do any operations on '%(cpv)s', since "
-		"it's EAPI is higher than this portage version's. Please upgrade"
+		"its EAPI is higher than this portage version's. Please upgrade"
 		" to a portage version that supports EAPI '%(eapi)s'.") % \
-		{"cpv": self.cpv, "eapi": str(self.eapi).lstrip("-")}
-		return msg
+		{"cpv": self.cpv, "eapi": eapi}
+		return _unicode_decode(msg,
+			encoding=_encodings['content'], errors='replace')
 
+	if sys.hexversion < 0x3000000:
 
+		__unicode__ = __str__
+
+		def __str__(self):
+			return _unicode_encode(self.__unicode__(),
+				encoding=_encodings['content'], errors='backslashreplace')
 
 class SignatureException(PortageException):
 	"""Signature was not present in the checked file"""
