@@ -1,17 +1,13 @@
-# Copyright 1998-2009 Gentoo Foundation
+# Copyright 1998-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Id: output.py 15245 2010-01-29 18:47:30Z zmedico $
 
 __docformat__ = "epytext"
 
-import codecs
-try:
-	from subprocess import getstatusoutput as subprocess_getstatusoutput
-except ImportError:
-	from commands import getstatusoutput as subprocess_getstatusoutput
 import errno
+import io
 import formatter
 import re
+import subprocess
 import sys
 
 import portage
@@ -139,6 +135,9 @@ _styles["PKG_BLOCKER_SATISFIED"]   = ( "darkblue", )
 _styles["PKG_MERGE"]               = ( "darkgreen", )
 _styles["PKG_MERGE_SYSTEM"]        = ( "darkgreen", )
 _styles["PKG_MERGE_WORLD"]         = ( "green", )
+_styles["PKG_BINARY_MERGE"]        = ( "purple", )
+_styles["PKG_BINARY_MERGE_SYSTEM"] = ( "purple", )
+_styles["PKG_BINARY_MERGE_WORLD"]  = ( "fuchsia", )
 _styles["PKG_UNINSTALL"]           = ( "red", )
 _styles["PKG_NOMERGE"]             = ( "darkblue", )
 _styles["PKG_NOMERGE_SYSTEM"]      = ( "darkblue", )
@@ -164,13 +163,13 @@ def _parse_color_map(config_root='/', onerror=None):
 		if token[0] in quotes and token[0] == token[-1]:
 			token = token[1:-1]
 		return token
-	try:
-		lineno=0
-		for line in codecs.open(_unicode_encode(myfile,
-			encoding=_encodings['fs'], errors='strict'),
-			mode='r', encoding=_encodings['content'], errors='replace'):
-			lineno += 1
 
+	try:
+		with io.open(_unicode_encode(myfile,
+			encoding=_encodings['fs'], errors='strict'),
+			mode='r', encoding=_encodings['content'], errors='replace') as f:
+			lines = f.readlines()
+		for lineno, line in enumerate(lines):
 			commenter_pos = line.find("#")
 			line = line[:commenter_pos].strip()
 			
@@ -179,7 +178,7 @@ def _parse_color_map(config_root='/', onerror=None):
 			
 			split_line = line.split("=")
 			if len(split_line) != 2:
-				e = ParseError(_("'%s', line %s: expected exactly one occurence of '=' operator") % \
+				e = ParseError(_("'%s', line %s: expected exactly one occurrence of '=' operator") % \
 					(myfile, lineno))
 				raise e
 				if onerror:
@@ -240,7 +239,7 @@ _max_xtermTitle_len = 253
 def xtermTitle(mystr, raw=False):
 	global _disable_xtermTitle
 	if _disable_xtermTitle is None:
-		_disable_xtermTitle = not (sys.stderr.isatty() and \
+		_disable_xtermTitle = not (sys.__stderr__.isatty() and \
 		'TERM' in os.environ and \
 		_legal_terms_re.match(os.environ['TERM']) is not None)
 
@@ -266,32 +265,36 @@ default_xterm_title = None
 def xtermTitleReset():
 	global default_xterm_title
 	if default_xterm_title is None:
-		prompt_command = os.getenv('PROMPT_COMMAND')
+		prompt_command = os.environ.get('PROMPT_COMMAND')
 		if prompt_command == "":
 			default_xterm_title = ""
 		elif prompt_command is not None:
 			if dotitles and \
 				'TERM' in os.environ and \
 				_legal_terms_re.match(os.environ['TERM']) is not None and \
-				sys.stderr.isatty():
+				sys.__stderr__.isatty():
 				from portage.process import find_binary, spawn
 				shell = os.environ.get("SHELL")
 				if not shell or not os.access(shell, os.EX_OK):
 					shell = find_binary("sh")
 				if shell:
 					spawn([shell, "-c", prompt_command], env=os.environ,
-						fd_pipes={0:sys.stdin.fileno(),1:sys.stderr.fileno(),
-						2:sys.stderr.fileno()})
+						fd_pipes={
+							0: portage._get_stdin().fileno(),
+							1: sys.__stderr__.fileno(),
+							2: sys.__stderr__.fileno()
+						})
 				else:
 					os.system(prompt_command)
 			return
 		else:
-			pwd = os.getenv('PWD','')
-			home = os.getenv('HOME', '')
+			pwd = os.environ.get('PWD','')
+			home = os.environ.get('HOME', '')
 			if home != '' and pwd.startswith(home):
 				pwd = '~' + pwd[len(home):]
 			default_xterm_title = '\x1b]0;%s@%s:%s\x07' % (
-				os.getenv('LOGNAME', ''), os.getenv('HOSTNAME', '').split('.', 1)[0], pwd)
+				os.environ.get('LOGNAME', ''),
+				os.environ.get('HOSTNAME', '').split('.', 1)[0], pwd)
 	xtermTitle(default_xterm_title, raw=True)
 
 def notitles():
@@ -320,6 +323,12 @@ def style_to_ansi_code(style):
 		ret += codes.get(attr_name, attr_name)
 	return ret
 
+def colormap():
+	mycolors = []
+	for c in ("GOOD", "WARN", "BAD", "HILITE", "BRACKET", "NORMAL"):
+		mycolors.append("%s=$'%s'" % (c, style_to_ansi_code(c)))
+	return "\n".join(mycolors)
+
 def colorize(color_key, text):
 	global havecolor
 	if havecolor:
@@ -336,12 +345,12 @@ compat_functions_colors = ["bold","white","teal","turquoise","darkteal",
 	"fuchsia","purple","blue","darkblue","green","darkgreen","yellow",
 	"brown","darkyellow","red","darkred"]
 
-def create_color_func(color_key):
-	def derived_func(*args):
-		newargs = list(args)
-		newargs.insert(0, color_key)
-		return colorize(*newargs)
-	return derived_func
+class create_color_func(object):
+	__slots__ = ("_color_key",)
+	def __init__(self, color_key):
+		self._color_key = color_key
+	def __call__(self, text):
+		return colorize(self._color_key, text)
 
 for c in compat_functions_colors:
 	globals()[c] = create_color_func(c)
@@ -414,33 +423,51 @@ class StyleWriter(formatter.DumbWriter):
 		if self.style_listener:
 			self.style_listener(styles)
 
-def get_term_size():
+def get_term_size(fd=None):
 	"""
 	Get the number of lines and columns of the tty that is connected to
-	stdout.  Returns a tuple of (lines, columns) or (-1, -1) if an error
+	fd.  Returns a tuple of (lines, columns) or (0, 0) if an error
 	occurs. The curses module is used if available, otherwise the output of
-	`stty size` is parsed.
+	`stty size` is parsed. The lines and columns values are guaranteed to be
+	greater than or equal to zero, since a negative COLUMNS variable is
+	known to prevent some commands from working (see bug #394091).
 	"""
-	if not sys.stdout.isatty():
-		return -1, -1
+	if fd is None:
+		fd = sys.stdout
+	if not hasattr(fd, 'isatty') or not fd.isatty():
+		return (0, 0)
 	try:
 		import curses
 		try:
-			curses.setupterm()
+			curses.setupterm(term=os.environ.get("TERM", "unknown"),
+				fd=fd.fileno())
 			return curses.tigetnum('lines'), curses.tigetnum('cols')
 		except curses.error:
 			pass
 	except ImportError:
 		pass
-	st, out = subprocess_getstatusoutput('stty size')
-	if st == os.EX_OK:
+
+	try:
+		proc = subprocess.Popen(["stty", "size"],
+			stdout=subprocess.PIPE, stderr=fd)
+	except EnvironmentError as e:
+		if e.errno != errno.ENOENT:
+			raise
+		# stty command not found
+		return (0, 0)
+
+	out = _unicode_decode(proc.communicate()[0])
+	if proc.wait() == os.EX_OK:
 		out = out.split()
 		if len(out) == 2:
 			try:
-				return int(out[0]), int(out[1])
+				val = (int(out[0]), int(out[1]))
 			except ValueError:
 				pass
-	return -1, -1
+			else:
+				if val[0] >= 0 and val[1] >= 0:
+					return val
+	return (0, 0)
 
 def set_term_size(lines, columns, fd):
 	"""
@@ -484,21 +511,13 @@ class EOutput(object):
 		lines, columns = get_term_size()
 		if columns <= 0:
 			columns = 80
-		# Adjust columns so that eend works properly on a standard BSD console.
-		if os.environ.get("TERM") in ("cons25", "dumb"):
-			columns = columns - 1
 		self.term_columns = columns
 		sys.stdout.flush()
 		sys.stderr.flush()
 
 	def _write(self, f, s):
 		# avoid potential UnicodeEncodeError
-		s = _unicode_encode(s,
-			encoding=_encodings['stdio'], errors='backslashreplace')
-		if sys.hexversion >= 0x3000000:
-			f = f.buffer
-		f.write(s)
-		f.flush()
+		writemsg(s, noiselevel=-1, fd=f)
 
 	def __eend(self, caller, errno, msg):
 		if errno == 0:
@@ -515,7 +534,7 @@ class EOutput(object):
 		if not self.quiet:
 			out = sys.stdout
 			self._write(out,
-				"%*s%s\n" % ((self.term_columns - self.__last_e_len - 6),
+				"%*s%s\n" % ((self.term_columns - self.__last_e_len - 7),
 				"", status_brackets))
 
 	def ebegin(self, msg):
@@ -529,7 +548,7 @@ class EOutput(object):
 		msg += " ..."
 		if not self.quiet:
 			self.einfon(msg)
-		self.__last_e_len = len(msg) + 4
+		self.__last_e_len = len(msg) + 3
 		self.__last_e_cmd = "ebegin"
 
 	def eend(self, errno, *msg):
@@ -623,11 +642,14 @@ class EOutput(object):
 class ProgressBar(object):
 	"""The interface is copied from the ProgressBar class from the EasyDialogs
 	module (which is Mac only)."""
-	def __init__(self, title=None, maxval=0, label=None):
-		self._title = title
+	def __init__(self, title=None, maxval=0, label=None, max_desc_length=25):
+		self._title = title or ""
 		self._maxval = maxval
-		self._label = maxval
+		self._label = label or ""
 		self._curval = 0
+		self._desc = ""
+		self._desc_max_length = max_desc_length
+		self._set_desc()
 
 	@property
 	def curval(self):
@@ -651,10 +673,23 @@ class ProgressBar(object):
 	def title(self, newstr):
 		"""Sets the text in the title bar of the progress dialog to newstr."""
 		self._title = newstr
+		self._set_desc()
 
 	def label(self, newstr):
 		"""Sets the text in the progress box of the progress dialog to newstr."""
 		self._label = newstr
+		self._set_desc()
+
+	def _set_desc(self):
+		self._desc = "%s%s" % (
+			"%s: " % self._title if self._title else "",
+			"%s" % self._label if self._label else ""
+		)
+		if len(self._desc) > self._desc_max_length:  # truncate if too long
+			self._desc = "%s..." % self._desc[:self._desc_max_length - 3]
+		if len(self._desc):
+			self._desc = self._desc.ljust(self._desc_max_length)
+
 
 	def set(self, value, maxval=None):
 		"""
@@ -683,10 +718,10 @@ class ProgressBar(object):
 
 class TermProgressBar(ProgressBar):
 	"""A tty progress bar similar to wget's."""
-	def __init__(self, **kwargs):
+	def __init__(self, fd=sys.stdout, **kwargs):
 		ProgressBar.__init__(self, **kwargs)
-		lines, self.term_columns = get_term_size()
-		self.file = sys.stdout
+		lines, self.term_columns = get_term_size(fd)
+		self.file = fd
 		self._min_columns = 11
 		self._max_columns = 80
 		# for indeterminate mode, ranges from 0.0 to 1.0
@@ -709,16 +744,18 @@ class TermProgressBar(ProgressBar):
 		curval = self._curval
 		maxval = self._maxval
 		position = self._position
-		percentage_str_width = 4
+		percentage_str_width = 5
 		square_brackets_width = 2
 		if cols < percentage_str_width:
 			return ""
-		bar_space = cols - percentage_str_width - square_brackets_width
+		bar_space = cols - percentage_str_width - square_brackets_width - 1
+		if self._desc:
+			bar_space -= self._desc_max_length
 		if maxval == 0:
 			max_bar_width = bar_space-3
-			image = "    "
+			_percent = "".ljust(percentage_str_width)
 			if cols < min_columns:
-				return image
+				return ""
 			if position <= 0.5:
 				offset = 2 * position
 			else:
@@ -734,16 +771,16 @@ class TermProgressBar(ProgressBar):
 				position = 0.5
 			self._position = position
 			bar_width = int(offset * max_bar_width)
-			image = image + "[" + (bar_width * " ") + \
-				"<=>" + ((max_bar_width - bar_width) * " ") + "]"
+			image = "%s%s%s" % (self._desc, _percent,
+				"[" + (bar_width * " ") + \
+				"<=>" + ((max_bar_width - bar_width) * " ") + "]")
 			return image
 		else:
 			percentage = int(100 * float(curval) / maxval)
-			if percentage == 100:
-				percentage_str_width += 1
-				bar_space -= 1
 			max_bar_width = bar_space - 1
-			image = ("%d%% " % percentage).rjust(percentage_str_width)
+			_percent = ("%d%% " % percentage).rjust(percentage_str_width)
+			image = "%s%s" % (self._desc, _percent)
+
 			if cols < min_columns:
 				return image
 			offset = float(curval) / maxval
