@@ -1,4 +1,4 @@
-# Copyright 2010-2012 Gentoo Foundation
+# Copyright 2010-2013 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
 
 __all__ = ['env_update']
@@ -171,9 +171,9 @@ def _env_update(makelinks, target_root, prev_mtimes, contents, env,
 		myld = io.open(_unicode_encode(ldsoconf_path,
 			encoding=_encodings['fs'], errors='strict'),
 			mode='r', encoding=_encodings['content'], errors='replace')
-		myldlines=myld.readlines()
+		myldlines = myld.readlines()
 		myld.close()
-		oldld=[]
+		oldld = []
 		for x in myldlines:
 			#each line has at least one char (a newline)
 			if x[:1] == "#":
@@ -194,20 +194,34 @@ def _env_update(makelinks, target_root, prev_mtimes, contents, env,
 			myfd.write(x + "\n")
 		myfd.close()
 
+	potential_lib_dirs = set()
+	for lib_dir_glob in ('usr/lib*', 'lib*'):
+		x = os.path.join(eroot, lib_dir_glob)
+		for y in glob.glob(_unicode_encode(x,
+			encoding=_encodings['fs'], errors='strict')):
+			try:
+				y = _unicode_decode(y,
+					encoding=_encodings['fs'], errors='strict')
+			except UnicodeDecodeError:
+				continue
+			if os.path.basename(y) != 'libexec':
+				potential_lib_dirs.add(y[len(eroot):])
+
 	# Update prelink.conf if we are prelink-enabled
 	if prelink_capable:
-		newprelink = atomic_ofstream(os.path.join(
-			eroot, "etc", "prelink.conf"))
+		prelink_d = os.path.join(eroot, 'etc', 'prelink.conf.d')
+		ensure_dirs(prelink_d)
+		newprelink = atomic_ofstream(os.path.join(prelink_d, 'portage.conf'))
 		newprelink.write("# prelink.conf autogenerated by env-update; make all changes to\n")
 		newprelink.write("# contents of /etc/env.d directory\n")
 
-		for x in ["/bin","/sbin","/usr/bin","/usr/sbin","/lib","/usr/lib"]:
-			newprelink.write("-l %s\n" % (x,));
-		prelink_paths = []
-		prelink_paths += specials.get("LDPATH", [])
-		prelink_paths += specials.get("PATH", [])
-		prelink_paths += specials.get("PRELINK_PATH", [])
-		prelink_path_mask = specials.get("PRELINK_PATH_MASK", [])
+		for x in sorted(potential_lib_dirs) + ['bin', 'sbin']:
+			newprelink.write('-l /%s\n' % (x,));
+		prelink_paths = set()
+		prelink_paths |= set(specials.get('LDPATH', []))
+		prelink_paths |= set(specials.get('PATH', []))
+		prelink_paths |= set(specials.get('PRELINK_PATH', []))
+		prelink_path_mask = specials.get('PRELINK_PATH_MASK', [])
 		for x in prelink_paths:
 			if not x:
 				continue
@@ -228,24 +242,25 @@ def _env_update(makelinks, target_root, prev_mtimes, contents, env,
 			newprelink.write("-b %s\n" % (x,))
 		newprelink.close()
 
+		# Migration code path.  If /etc/prelink.conf was generated by us, then
+		# point it to the new stuff until the prelink package re-installs.
+		prelink_conf = os.path.join(eroot, 'etc', 'prelink.conf')
+		try:
+			with open(_unicode_encode(prelink_conf,
+				encoding=_encodings['fs'], errors='strict'), 'rb') as f:
+				if f.readline() == b'# prelink.conf autogenerated by env-update; make all changes to\n':
+					f = atomic_ofstream(prelink_conf)
+					f.write('-c /etc/prelink.conf.d/*.conf\n')
+					f.close()
+		except IOError as e:
+			if e.errno != errno.ENOENT:
+				raise
+
 	current_time = long(time.time())
 	mtime_changed = False
 
-	potential_lib_dirs = []
-	for lib_dir_glob in ['usr/lib*', 'lib*']:
-		x = os.path.join(eroot, lib_dir_glob)
-		for y in glob.glob(_unicode_encode(x,
-			encoding=_encodings['fs'], errors='strict')):
-			try:
-				y = _unicode_decode(y,
-					encoding=_encodings['fs'], errors='strict')
-			except UnicodeDecodeError:
-				continue
-			if os.path.basename(y) != "libexec":
-				potential_lib_dirs.append(y[len(eroot):])
-
 	lib_dirs = set()
-	for lib_dir in set(specials["LDPATH"] + potential_lib_dirs):
+	for lib_dir in set(specials['LDPATH']) | potential_lib_dirs:
 		x = os.path.join(eroot, lib_dir.lstrip(os.sep))
 		try:
 			newldpathtime = os.stat(x)[stat.ST_MTIME]
@@ -306,7 +321,7 @@ def _env_update(makelinks, target_root, prev_mtimes, contents, env,
 			writemsg_level(_(">>> Regenerating %setc/ld.so.cache...\n") % \
 				(target_root,))
 			os.system("cd / ; %s -X -r '%s'" % (ldconfig, target_root))
-		elif ostype in ("FreeBSD","DragonFly"):
+		elif ostype in ("FreeBSD", "DragonFly"):
 			writemsg_level(_(">>> Regenerating %svar/run/ld-elf.so.hints...\n") % \
 				target_root)
 			os.system(("cd / ; %s -elf -i " + \
@@ -325,7 +340,7 @@ def _env_update(makelinks, target_root, prev_mtimes, contents, env,
 	outfile = atomic_ofstream(os.path.join(eroot, "etc", "profile.env"))
 	outfile.write(penvnotice)
 
-	env_keys = [ x for x in env if x != "LDPATH" ]
+	env_keys = [x for x in env if x != "LDPATH"]
 	env_keys.sort()
 	for k in env_keys:
 		v = env[k]
